@@ -1,7 +1,6 @@
 '''Interfaces for supported data-structures'''
 
-from stdnet.utils import listPipeline, mapPipeline
-
+from stdnet.utils import iteritems
 
 __all__ = ['PipeLine',
            'pipelines',
@@ -14,6 +13,25 @@ __all__ = ['PipeLine',
 default_score = lambda x : 1
 
 
+
+class listPipeline(object):
+    def __init__(self):
+        self.clear()
+        
+    def push_front(self, value):
+        self.front.append(value)
+        
+    def push_back(self, value):
+        self.back.append(value)
+        
+    def clear(self):
+        self.back = []
+        self.front = []
+        
+    def __len__(self):
+        return len(self.back) + len(self.front)
+
+
 class keyconverter(object):
     
     @classmethod
@@ -39,9 +57,9 @@ class HashPipe(PipeLine):
     def __init__(self, timeout):
         super(HashPipe,self).__init__({},'hash',timeout)
         
-class MapPipe(PipeLine):
+class TsPipe(PipeLine):
     def __init__(self, timeout):
-        super(MapPipe,self).__init__(mapPipeline(),'map',timeout)
+        super(TsPipe,self).__init__({},'ts',timeout)
 
 class SetPipe(PipeLine):
     def __init__(self, timeout):
@@ -58,7 +76,7 @@ class ListPipe(PipeLine):
 
 _pipelines = {'list':ListPipe,
               'hash': HashPipe,
-              'map': MapPipe,
+              'ts': TsPipe,
               'set': SetPipe,
               'oset': OsetPipe}
 
@@ -272,13 +290,6 @@ class OrderedSet(Set):
         self.pipeline.add((value.score(),self.pickler.dumps(value)))
 
 
-def itemcmp(x,y):
-    if x[0] > y[0]:
-        return 1
-    else:
-        return -1
-
-
 class KeyValueStructure(Structure):
     '''Base class for :class:`HashTable`'''
     def __init__(self, *args, **kwargs):
@@ -295,15 +306,18 @@ class KeyValueStructure(Structure):
     
     
 class HashTable(Structure):
-    '''A hash-table :class:`stdnet.Structure`. Equivalent to a Python ``dict``.
+    '''A hash-table :class:`stdnet.Structure`.
+The networked equivalent to a Python ``dict``.
     
-This structure is used for in two different parts of the library.
+This structure is important since it is used in two different parts of the library.
 
-* It is the structure which holds instances for a :class:`stdnet.orm.StdModel` class. Therefore each
-  model is represented as a HashTable structure. The keys are the model instances ids and the values are the 
+* It is the structure which holds instances for a :class:`stdnet.orm.StdModel` class.
+  Therefore each model is represented as a HashTable structure.
+  The keys are the model instances ids and the values are the 
   serialised version of the instances.
 
-* It is also used as :class:`stdnet.orm.HashField`.'''
+* It is also used as field (:class:`stdnet.orm.HashField`) like all other 
+  class:`stdnet.Structure`.'''
     struct = HashPipe
     def __init__(self, *args, **kwargs):
         self.converter = kwargs.pop('converter',None) or keyconverter
@@ -326,7 +340,7 @@ This structure is used for in two different parts of the library.
         tokey = self.converter.tokey
         dumps = self.pickler.dumps
         p     = self.pipeline
-        for key,value in mapping.iteritems():
+        for key,value in iteritems(mapping):
             p[tokey(key)] = dumps(value)
     
     def get(self, key, default = None):
@@ -349,8 +363,7 @@ This structure is used for in two different parts of the library.
         if not keys:
             raise StopIteration
         tokey = self.converter.tokey
-        ckeys = [tokey(key) for key in keys]
-        objs  = self._mget(ckeys)
+        objs  = self._mget([tokey(key) for key in keys])
         loads = self.pickler.loads
         for obj in objs:
             yield loads(obj)
@@ -382,7 +395,7 @@ This structure is used for in two different parts of the library.
         return keys
             
     def sorteditems(self, desc = True):
-        items = sorted(self.items(),cmp = itemcmp)
+        items = sorted(self.items(),key = lambda t : t[0])
         if not desc:
             items = reversed(items)
         return items
@@ -409,21 +422,15 @@ This structure is used for in two different parts of the library.
         raise NotImplementedError
 
     
-class Map(HashTable):
-    struct = MapPipe
-    def __init__(self, *args, **kwargs):
-        self.scorefun = kwargs.pop('scorefun',default_score)
-        super(Map,self).__init__(*args, **kwargs)
+class TS(HashTable):
+    struct = TsPipe
         
     def update(self, mapping):
-        '''Add *mapping* dictionary to hashtable. Equivalent to python dictionary update method.'''
         tokey = self.converter.tokey
         dumps = self.pickler.dumps
-        sfunc = self.scorefun
         p     = self.pipeline
-        for key,value in mapping.iteritems():
-            rk = tokey(key)
-            p[rk] = dumps(value)
+        for key,value in iteritems(mapping):
+            p[tokey(key)] = dumps(value)
     
     def front(self):
         try:
@@ -441,15 +448,13 @@ class Map(HashTable):
         '''Return a range between start and end key.'''
         tokey    = self.converter.tokey
         tovalue  = self.converter.tovalue
-        sfunc    = self.scorefun
         loads    = self.pickler.loads
-        for key,val in self._range(sfunc(tokey(start)),sfunc(tokey(end))):
+        for key,val in self._range(tokey(start),tokey(end)):
             yield tovalue(key),loads(val)
             
     def count(self, start, end):
         tokey    = self.converter.tokey
-        sfunc    = self.scorefun
-        return self._count(sfunc(tokey(start)),sfunc(tokey(end)))
+        return self._count(tokey(start),tokey(end))
             
     def irange(self, start = 0, end = -1):
         '''Return a range between start and end key.'''
