@@ -3,17 +3,14 @@ from hashlib import sha1
 import time
 from datetime import date, datetime
 
-from query import RelatedManager
-from related import RelatedObject, ReverseSingleRelatedObjectDescriptor
 from stdnet.exceptions import *
-from stdnet.utils import timestamp2date, date2timestamp
+from stdnet.utils import pickle, json, json_compact, DefaultJSONEncoder,\
+                         DefaultJSONHook, timestamp2date, date2timestamp,\
+                         UnicodeMixin, novalue, to_string, is_string
 
-try:
-    import cPickle as pickle
-except ImportError:
-    import pickle
-
-hashfun = lambda x : sha1(x).hexdigest()
+from .related import RelatedObject, ReverseSingleRelatedObjectDescriptor
+from .query import RelatedManager
+from .globals import get_model_from_hash
 
 
 __all__ = ['Field',
@@ -27,16 +24,12 @@ __all__ = ['Field',
            'SymbolField',
            'CharField',
            'ForeignKey',
+           'JSONField',
            'PickleObjectField',
-           '_novalue']
-
-class NoValue(object):
-    pass
-
-_novalue = NoValue()
+           'ModelField']
 
 
-class Field(object):
+class Field(UnicodeMixin):
     '''This is the base class of all StdNet Fields.
 Each field is specified as a :class:`stdnet.orm.StdModel` class attribute.
     
@@ -80,7 +73,7 @@ Each field is specified as a :class:`stdnet.orm.StdModel` class attribute.
 
     Default value for this field.
     
-    Default :class:`NoValue`.
+    Default ``novalue``.
     
 .. attribute:: name
 
@@ -91,10 +84,10 @@ Each field is specified as a :class:`stdnet.orm.StdModel` class attribute.
     The :class:`stdnet.orm.StdModel` holding the field.
     Created by the ``orm`` at runtime. 
 '''
-    default=NoValue
+    default=novalue
     
     def __init__(self, unique = False, ordered = False, primary_key = False,
-                 required = True, index = True, default=NoValue):
+                 required = True, index = True, default=novalue, **extras):
         self.primary_key = primary_key
         if primary_key:
             self.unique   = True
@@ -108,13 +101,19 @@ Each field is specified as a :class:`stdnet.orm.StdModel` class attribute.
         self.meta     = None
         self.name     = None
         self.model    = None
-        self.default  = default if default is not NoValue else self.default
+        self.default  = default if default is not novalue else self.default
+        self._handle_extras(**extras)
         
-    def __str__(self):
-        return '%s.%s' % (self.meta,self.name)
-    
-    def __repr__(self):
-        return '%s: %s' % (self.__class__.__name__,self)
+    def _handle_extras(self, **extras):
+        self.error_extras(extras)
+        
+    def error_extras(self, extras):
+        keys = list(extras)
+        if keys:
+            raise TypeError("__init__() got an unexepcted keyword argument '{0}'".format(keys[0]))
+        
+    def __unicode__(self):
+        return to_string('%s.%s' % (self.meta,self.name))
         
     def to_python(self, value):
         """Converts the input value into the expected Python
@@ -164,17 +163,21 @@ If an error occurs it raises :class:`stdnet.exceptions.FieldValueError`'''
     
     def has_default(self):
         "Returns a boolean of whether this field has a default value."
-        return self.default is not NoValue
+        return self.default is not novalue
     
     def get_default(self):
         "Returns the default value for this field."
         if self.has_default():
-            if callable(self.default):
+            if hasattr(self.default,'__call__'):
                 return self.default()
             else:
                 return self.default
         return None
     
+    def index_value(self):
+        '''A value which is used by indexes to generate keys.'''
+        return self.value
+    
     def __deepcopy__(self, memodict):
         '''Nothing to deepcopy here'''
         field = copy(self)
@@ -195,24 +198,32 @@ value with a specific data type. it can be of four different types:
 * floating point
 * symbol
 '''
-    type = None
-
+    pass
+        
+    
 
 class SymbolField(AtomField):
     '''An :class:`AtomField` which contains a ``symbol``.
 A symbol holds a sequence of characters as a single unit.
 A symbol is irreducible, and are often used to hold names, codes
-or other entities.'''
+or other entities. They are indexes by default.'''
     type = 'symbol'
-    def serialize(self, value):
+    default = to_string('')
+    
+    def to_python(self, value):
         if value is not None:
-            value = str(value)
-        return value
-
+            return to_string(value)
+        else:
+            return self.default
+    
+    def index_value(self):
+        return sha1(self.value)
+    
 
 class IntegerField(AtomField):
     '''An integer :class:`AtomField`.'''
     type = 'integer'
+    default = 0
     def serialise(self, value):
         if value is not None:
             try:
@@ -221,6 +232,13 @@ class IntegerField(AtomField):
                 raise FieldValueError('Field is not a valid integer')
         return value
     
+    def to_python(self, value):
+        if value:
+            return int(value)
+        else:
+            return self.default
+        
+    
     
 class BooleanField(AtomField):
     '''An boolean :class:`AtomField`'''
@@ -228,6 +246,12 @@ class BooleanField(AtomField):
     def serialise(self, value):
         return True if value else False
         
+    def to_python(self, value):
+        return True if value else False
+    
+    def index_value(self):
+        return 1 if self.value else 0
+    
     
 class AutoField(IntegerField):
     '''An :class:`IntegerField` that automatically increments.
@@ -247,6 +271,7 @@ class FloatField(AtomField):
 its :attr:`Field.index` is set to ``False``.
     '''
     type = 'float'
+    default = 0.
     def __init__(self,*args,**kwargs):
         index = kwargs.get('index',None)
         if index is None:
@@ -261,11 +286,18 @@ its :attr:`Field.index` is set to ``False``.
                 raise FieldValueError('Field is not a valid float')
         return value
     
+    def to_python(self, value):
+        if value:
+            return float(value)
+        else:
+            return self.default
+    
     
 class DateField(AtomField):
     '''An date :class:`AtomField` represented in Python by
 a :class:`datetime.date` instance.'''
     type = 'date'
+    default = None
     def serialize(self, value):
         if value is not None:
             if isinstance(value,date):
@@ -291,17 +323,17 @@ a :class:`datetime.datetime` instance.'''
         return value
 
 
-class CharField(Field):
+class CharField(SymbolField):
     '''A text :class:`Field` which is never an index.
 It contains strings and by default :attr:`Field.required`
 is set to ``False``.'''
-    default = ''
     type = 'text'
     def __init__(self, *args, **kwargs):
         kwargs['index'] = False
         kwargs['unique'] = False
         kwargs['primary_key'] = False
         kwargs['ordered'] = False
+        self.max_length = kwargs.pop('max_length',None) # not used for now 
         required = kwargs.get('required',None)
         if required is None:
             kwargs['required'] = False
@@ -367,4 +399,40 @@ back to self. For example::
             return value.id
         except:
             return value
-    
\ No newline at end of file
+    
+    
+class JSONField(CharField):
+    '''A JSON field which implements authomatic converion to and form dictionary of data.'''
+    type = 'json object'
+    def __init__(self, *args, **kwargs):
+        kwargs['default'] = kwargs.get('default',{})
+        self.encoder_class = kwargs.pop('encoder_class',DefaultJSONEncoder)
+        self.decoder_hook  = kwargs.pop('decoder_hook',DefaultJSONHook)
+        self.sep = kwargs.pop('sep',None)
+        super(JSONField,self).__init__(*args, **kwargs)
+        
+    def to_python(self, value):
+        if is_string(value):
+            if not value:
+                value = {}
+            else:
+                value = json.loads(value, object_hook = self.decoder_hook)
+        return value
+    
+    def serialize(self, value):
+        if value is not None:
+            value = json.dumps(json_compact(value,self.sep), cls=self.encoder_class)
+        return value
+    
+
+class ModelField(SymbolField):
+    '''A filed which can be used to store the model unique sha1'''
+    
+    def to_python(self, value):
+        return get_model_from_hash(value)
+    
+    def serialize(self, value):
+        if value is not None:
+            value = value._meta.hash
+        return value
+
