@@ -1,10 +1,19 @@
-from fields import Field, RelatedObject, _novalue
-
 from stdnet.exceptions import *
 from stdnet import pipelines
 from stdnet.orm.related import add_lazy_relation, ModelFieldPickler
 
-from query import M2MRelatedManager
+from .fields import Field, RelatedObject
+from .query import M2MRelatedManager
+
+
+__all__ = ['ManyFieldManagerProxy',
+           'Many2ManyManagerProxy',
+           'MultiField',
+           'SetField',
+           'ListField',
+           'HashField',
+           #'TSField',
+           'ManyToManyField']
 
 
 class ManyFieldManagerProxy(object):
@@ -19,9 +28,6 @@ class ManyFieldManagerProxy(object):
     def get_cache_name(self):
         return '_%s_cache' % self.name
     
-    def id(self, instance):
-        return instance._meta.basekey('id',instance.id,self.name)
-
     def __get__(self, instance, instance_type=None):
         if instance is None:
             return self
@@ -124,7 +130,10 @@ class MultiField(Field):
         self.model._meta.multifields.append(self)
         
     def to_python(self, instance):
-        return None    
+        return None
+    
+    def id(self, obj):
+        return getattr(obj,self.attname).id
 
 
 class SetField(MultiField):
@@ -189,16 +198,6 @@ it returns an instance of :class:`stdnet.HashTable` structure.
     type = 'hash'
     def get_pipeline(self):
         return 'hash'
-    
-
-class MapField(MultiField):
-    '''A Hash table field, the networked equivalent of a python dictionary.
-Keys are string while values are string/numeric.
-it returns an instance of :class:`stdnet.HashTable` structure.
-'''
-    type = 'map'
-    def get_pipeline(self):
-        return 'map'
 
 
 class ManyToManyField(MultiField):
